library(lubridate)
library(startR)
library(s2dverification)
library(multiApply)

obs_path <- '/esarchive/recon/ecmwf/era5/monthly_mean/$var$_f1h/$var$_$sdate$.nc'
pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')
y1 <- 1993
y2 <- 1995
var_name<-'sfcWind'

lons.min <- 0
lons.max <- 10
lats.min <- 0
lats.max <- 10

#-------------------------------
# Interpolation
#-------------------------------

obs <- Start(dat = obs_path,
             var = var_name,
#             sdate = paste0(y1:y2, '12'),
             sdate = '201811',
             time = 'all',
             latitude = values(list(lats.min, lats.max)),
             latitude_reorder = Sort(),  #obs3 without this line
             longitude = values(list(lons.min, lons.max)),
             synonims = list(longitude = c('lon', 'longitude'),
                             latitude = c('lat', 'latitude')),
             transform = CDORemapper,
             transform_params = list(grid = 'r360x181',
                                     method = 'conservative',
                                     crop = c(lons.min, lons.max, lats.min, lats.max)), #obs2 without this line
             transform_vars = c('latitude', 'longitude'),
             return_vars = list(time = NULL,
                                latitude = 'dat',
                                longitude = 'dat'),
             split_multiselected_dims = TRUE,
             merge_across_dims = TRUE,
             num_procs = 4,
             retrieve = T)

# load + interpolate data s2dverification
obs1 <- Load(var = var_name,
             obs = list(list(path = pobs)),
             sdates = '20181101',
#             sdates = paste0(y1:y2, '1201'),
             leadtimemin = 1,
             leadtimemax = 1,
             output = 'lonlat',
             grid = 'r360x181',
             method = 'conservative',
             storefreq = 'monthly',
             latmin = lats.min,
             latmax = lats.max,
             lonmin = lons.min,
             lonmax = lons.max)


## use: cdo sellonlatbox,0,10,0,10 -remapcon,r360x181 sfcWind_201811_regrid.nc sfcWind_201811_regrid_sub2.nc

library(easyNCDF)
  path <- '/home/Earth/aho/aho-testtest/startR/issue/issue37/sfcWind_201811_regrid_sub2.nc'
  fnc <- NcOpen(path)
  fnc_dims <- NcReadDims(fnc)
  fnc_arr <- NcToArray(fnc, vars_to_read = var_name)


##Compare
#--Start--obs--------------------------------------------------
dim(obs)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        11        11 
range(obs)
#[1] 0.9431505 4.7718749
str(attr(obs, 'Variables')$common[[var_name]]$dim[[1]])
# $ name         : chr "longitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
str(attr(obs, 'Variables')$common[[var_name]]$dim[[2]])
# $ name         : chr "latitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
obs[1,1,1,1,,1]
# [1] 4.771875 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304
# [9] 1.793729 1.684370 1.790771


#--Start--obs2-------------------------------------------------
dim(obs2)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        10        10 
range(obs2)
#[1] 1.177485 4.280102
str(attr(obs2, 'Variables')$common[[var_name]]$dim[[1]])
# $ name         : chr "longitude"
# $ vals         : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
str(attr(obs2, 'Variables')$common[[var_name]]$dim[[2]])
# $ name         : chr "latitude"
# $ vals         : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10
obs2[1,1,1,1,,1]
# [1] 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304 1.793729
# [9] 1.684370 1.790771  #consistent with obs, just miss lat=0

#--Start--obs3-------------------------------------------------
dim(obs3)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        11        11 
range(obs3)
#[1] 0.9431505 4.9703207
str(attr(obs3, 'Variables')$common[[var_name]]$dim[[1]])
# $ name         : chr "longitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
str(attr(obs3, 'Variables')$common[[var_name]]$dim[[2]])
# $ name         : chr "latitude"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
obs3[1,1,1,1,,1]
# [1] 4.970321 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304
# [9] 1.793729 1.684370 1.789749  #lat=0 and lat=11 different from obs

#--Load--------------------------------------------------------
dim(obs1$obs)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1      11      11 
range(obs1$obs)
#[1] 1.0905 4.9876
str(obs1)
# $ lon            : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
# $ lat            : num [1:11(1d)] 10 9 8 7 6 5 4 3 2 1 ...
obs1$obs[1,1,1,1,,1]  #when lon = 0
# [1] 1.8161 1.7187 1.8010 1.7741 2.2244 3.9090 3.6739 3.7585 4.0073 4.2994
#[11] 4.9876

#--cdo----------------------------------------------------------
dim(fnc_arr)
# var  lon  lat time 
#   1   11   11    1 
range(fnc_arr)
#[1] 1.090472 4.987585
str(attr(fnc_arr, 'variables')[[var_name]]$dim[[1]])
# $ name         : chr "lon"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
str(attr(fnc_arr, 'variables')[[var_name]]$dim[[2]])
# $ name         : chr "lat"
# $ vals         : num [1:11(1d)] 0 1 2 3 4 5 6 7 8 9 ...
fnc_arr[1,1,,1]
# [1] 4.987585 4.299388 4.007286 3.758497 3.673881 3.908989 2.224407 1.774090
# [9] 1.800998 1.718733 1.816074

##diff
diff <- drop(obs)[11:1, ] - drop(obs1$obs)
range(diff)
#[1] -0.2157251  0.8776323

fnc_arr2 <- s2dverification:::.aperm2(fnc_arr, c(4,3,2,1))
diff2 <- drop(fnc_arr2) - drop(obs)
range(diff2)
#[1] -0.8776584  0.2157096

diff3 <- drop(fnc_arr2)[11:1, ] - drop(obs1$obs)
range(diff3)
#[1] -4.910541e-05  4.688785e-05





#---------------------------------------
# Read in data first then s2dv::CDORemap
#---------------------------------------

obs_start <- Start(dat = obs_path,
                   var = var_name,
                   sdate = '201811',
#                   sdate = paste0(y1:y2, '12'),
                   time = 'all',
                   latitude = values(list(lats.min, lats.max)),
                   latitude_reorder = Sort(),
                   longitude = values(list(lons.min, lons.max)),
                   synonims = list(longitude = c('lon', 'longitude'),
                                   latitude = c('lat', 'latitude')),
                   return_vars = list(time = NULL,
                                      latitude = 'dat',
                                      longitude = 'dat'),
                   split_multiselected_dims = TRUE,
                   merge_across_dims = TRUE,
                   num_procs = 4,
                   retrieve = T)

#use s2dv::CDORemap to interpolate
lons <- as.vector((attr(obs_start, 'Variables')$dat1$longitude))
lats <- as.vector((attr(obs_start, 'Variables')$dat1$latitude))
obs_start_2 <- s2dverification::CDORemap(obs_start, lons, lats, 'r360x181', 'con', TRUE)

#--Start--obs--------------------------------------------------
dim(obs_start_2$data_array)
#      dat       var     sdate      time  latitude longitude 
#        1         1         1         1        10        10 
range(obs_start_2$data_array)
#[1] 1.177485 4.280102
str(obs_start_2)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10
obs_start_2$data_array[1,1,1,1,,1]
# [1] 4.280102 4.002453 3.771909 3.705606 3.982399 2.499812 1.827304 1.793729
# [9] 1.684370 1.790771  #same as obs

#----------------------------------------------------------------

pobs <- paste0('/esarchive/recon/ecmwf/era5/monthly_mean/',
               '$VAR_NAME$_f1h/$VAR_NAME$_$YEAR$$MONTH$.nc')

obs_s2dv <- Load(var = var_name,
                 obs = list(list(path = pobs)),
#                 sdates = paste0(y1:y2, '1201'),
                 sdates = '20181101',
                 leadtimemin = 1,
                 leadtimemax = 1,
                 output = 'lonlat',
                 storefreq = 'monthly',
                 latmin = lats.min,
                 latmax = lats.max,
                 lonmin = lons.min,
                 lonmax = lons.max)

#use s2dv::CDORemap to interpolate
lons <- as.vector(obs_s2dv$lon)
lats <- as.vector(obs_s2dv$lat)  #from 9.9765777 to 0.1405152!!
obs_s2dv_2 <- s2dverification::CDORemap(obs_s2dv$obs, lons, lats, 'r360x181', 'con', TRUE)

#--Load--------------------------------------------------------
dim(obs_s2dv_2$data_array)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1       9      10 
range(obs_s2dv_2$data_array)
#[1] 1.177490 4.280105
str(obs_s2dv_2)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:9(1d)] 1 2 3 4 5 6 7 8 9

obs_s2dv_2$data_array[1,1,1,1,,1]  #when lon = 0
# [1] 4.280105 4.002460 3.771910 3.705595 3.982394 2.499798 1.827297 1.793722
# [9] 1.684371  #similar to obs but a bit different

#--------------------------------------------------------------

obs_s2dv_3 <- s2dverification::CDORemap(obs_s2dv$obs, lons, rev(lats), 'r360x181', 'con', TRUE)
#--Load--------------------------------------------------------
dim(obs_s2dv_3$data_array)
#dataset  member   sdate   ftime     lat     lon 
#      1       1       1       1      10      10 
range(obs_s2dv_3$data_array)
#[1] 1.174429 4.728291
str(obs_s2dv_3)
# $ lons      : num [1:10(1d)] 0 1 2 3 4 5 6 7 8 9
# $ lats      : num [1:10(1d)] 1 2 3 4 5 6 7 8 9 10

obs_s2dv_3$data_array[1,1,1,1,,1]  #when lon = 0
# [1] 1.703802 1.760790 1.825117 2.210783 4.055306 3.708123 3.760025 3.971781
# [9] 4.228549 4.728291

#--------------------------------------------------------------









range(obs_start)
range(obs_s2dv$obs)
diff1 <- drop(obs_start) - drop(obs_s2dv$obs)[, 36:1, ]

